//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Device/Mask/MaskStack.h
//! @brief     Defines class MaskStack.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifdef SWIG
#error no need to expose this header to Swig
#endif
#ifndef BORNAGAIN_DEVICE_MASK_MASKSTACK_H
#define BORNAGAIN_DEVICE_MASK_MASKSTACK_H

#include "Base/Types/CloneableVector.h"
#include <memory>

class IShape2D;
class Frame;

class MaskPattern {
public:
    MaskPattern(IShape2D* shape_, bool doMask_, bool in_bins_);
    MaskPattern(const MaskPattern&) = delete;
    ~MaskPattern();
    MaskPattern* clone() const;
    IShape2D* shape; // cloneable
    bool doMask;
    bool inBins;
};

//! Collection of detector masks.

class MaskStack {
public:
    //! Add mask to the stack of detector masks.
    //! Argument mask_value=true means that the area will be excluded from the analysis.
    void maskToStack(const IShape2D& shape, bool mask_value, bool in_bins);

    bool isMasked(size_t i_flat, const Frame& frame) const;

    //! Returns true if has masks
    bool hasMasks() const;

    size_t numberOfMasks() const;

    const MaskPattern* patternAt(size_t iMask) const;

private:
    CloneableVector<MaskPattern> m_stack;
};

#endif // BORNAGAIN_DEVICE_MASK_MASKSTACK_H
