//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Data/DataAccessWidget.h
//! @brief     Defines class DataAccessWidget
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2022
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEW_DATA_DATAACCESSWIDGET_H
#define BORNAGAIN_GUI_VIEW_DATA_DATAACCESSWIDGET_H

#include <QWidget>

class Data1DItem;
class Data2DItem;
class JobItem;
class RealItem;

//! The DataAccessWidget class is a base for widgets, working data, stored in JobItem or RealItem.
//! DataAccessWidget provides an access to SpeculatDataItems and Data2DItems separately and
//! as a list of items that is required for simultaneous, synchronous work while plotting and
//! changing their properties.

class DataAccessWidget : public QWidget {
    Q_OBJECT
public:
    explicit DataAccessWidget(QWidget* parent = nullptr);
    ~DataAccessWidget() override;

    virtual void setJobOrRealItem(QObject* item) = 0;
    void setBaseItem(QObject* item);

    virtual void setFourierItem(Data2DItem* fftItem);

    virtual QList<QAction*> actionList();

    JobItem* jobItem() const;
    RealItem* realItem() const;
    QObject* jobRealBase() const;

    //... Access to concrete items:

    Data1DItem* realData1DItem() const;
    Data2DItem* realData2DItem() const;

    Data1DItem* simuData1DItem() const;
    Data2DItem* simuData2DItem() const;

    Data1DItem* diffData1DItem() const;
    Data2DItem* diffData2DItem() const;

    //... Access to lists of items:

    QList<Data1DItem*> mainData1DItems() const;
    QList<Data2DItem*> mainData2DItems() const;

    QList<Data1DItem*> allData1DItems() const;
    QList<Data2DItem*> allData2DItems() const;

    Data1DItem* currentData1DItem() const;
    Data2DItem* currentData2DItem() const;

private:
    QObject* m_item;
    Data2DItem* m_fftItem;
};

#endif // BORNAGAIN_GUI_VIEW_DATA_DATAACCESSWIDGET_H
