//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Item/RealDataStackPresenter.h
//! @brief     Defines class RealDataStackPresenter
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEW_ITEM_REALDATASTACKPRESENTER_H
#define BORNAGAIN_GUI_VIEW_ITEM_REALDATASTACKPRESENTER_H

#include "GUI/View/Item/ItemStackWidget.h"
#include <QMap>

class RealDataPresenter;

class RealDataStackPresenter : public ItemStackWidget {
public:
    //! Shows the widget for given item (and hides previous one).
    //! If no widget yet exists, it will be created.
    void setItem(QObject* item, QObject* model = nullptr);

    RealDataPresenter* itemWidget(QObject* item);

private:
    QMap<QObject*, RealDataPresenter*> m_itemToWidget;
};

//... Implementation.

void RealDataStackPresenter::setItem(QObject* item, QObject* model)
{
    if (!item) {
        hideWidgets();
        return;
    }

    RealDataPresenter* widget = itemWidget(item);

    if (!widget) {
        widget = new RealDataPresenter();
        m_stackedWidget->addWidget(widget);
        m_itemToWidget[item] = widget;
    }

    m_stackedWidget->setCurrentWidget(widget);
    if (widget->isHidden())
        widget->show();

    widget->setModel(model);
    widget->setJobOrRealItem(item);
}

RealDataPresenter* RealDataStackPresenter::itemWidget(QObject* /*item*/)
{
    if (!m_itemToWidget.empty())
        return m_itemToWidget.first();
    return nullptr;
}

#endif // BORNAGAIN_GUI_VIEW_ITEM_REALDATASTACKPRESENTER_H
